/*
 * Enhydra Java Application Server
 * The Initial Developer of the Original Code is Lutris Technologies Inc.
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc.
 * All Rights Reserved.
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License.
 *
 * $Id: OtterProject.java,v 1.1.2.7 2000/02/13 03:41:10 pmahar Exp $
 */

package org.enhydra.ide.jbuilder.common.node;

import org.enhydra.ide.jbuilder.common.PropUtil;
import java.io.File;
import java.util.Vector;

/**
 * Class declaration
 *
 *
 * @author
 * @version %I%, %G%
 */
abstract public class OtterProject implements OtterNode {
    private final String XMLC_COMPILE_JAVA = "enhydra.xmlc.compileJava";
    private final String XMLC_KEEP_JAVA = "enhydra.xmlc.keepJava";
    private final String XMLC_DUMP = "enhydra.xmlc.dump";
    private final String XMLC_PARSEINFO = "enhydra.xmlc.parseInfo";
    private final String XMLC_PRINTINFO = "enhydra.xmlc.printInfo";
    private final String XMLC_VERBOSE = "enhydra.xmlc.verbose";
    private final String XMLC_PRINTACCESSMETHODS = "enhydra.xmlc.printAccessMethods";
    private final String XMLC_SAVE_OUTPUT = "enhydra.xmlc.saveOutputToFile";
    private final String XMLC_OUTPUT_FILENAME = "enhydra.xmlc.outputToFilename";
    private final String XMLC_MAP_PACKAGE_FROM = "enhydra.xmlc.map.package.fromFolder";
    private final String XMLC_MAP_PACKAGE_TO = "enhydra.xmlc.map.package.toPackage";
    private final String XMLC_MAP_SCOPE = "enhydra.xmlc.map.scope";
    public final static int MAP_SCOPE_ALL      = 0;
    public final static int MAP_SCOPE_SELECTED = 1;
    public final static int MAP_SCOPE_NONE     = 2;


    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    abstract public Object getNativeProject();

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    abstract public String getName();

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    abstract public boolean isCompileJavaAllowed();

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    abstract public String getOutputDirectory();

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    abstract public String getSourcePath();

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    abstract public String[] getSourcePathArray();

    /**
     * Method declaration
     *
     *
     * @param pathAsArray
     *
     * @see
     */
    abstract public void setSourcePathArray(String[] pathAsArray);

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    abstract public String getClassPath();

    /**
     * Method declaration
     *
     *
     * @param property
     *
     * @return
     *
     * @see
     */
    abstract public String getProperty(String property);

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    abstract public OtterNodeFactory getNodeFactory();

    /**
     * Method declaration
     *
     *
     * @param property
     * @param value
     *
     * @see
     */
    abstract public void setProperty(String property, String value);
    abstract public void setProperty(String property, int value);

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public boolean isCompileJavaSelected() {
	boolean b = false;

	if (isCompileJavaAllowed()) {
	    b = readBooleanProperty(XMLC_COMPILE_JAVA, true);
	}

	return b;
    }

    /**
     * Method declaration
     *
     *
     * @param b
     *
     * @see
     */
    public void setCompileJavaSelected(boolean b) {
	writeBooleanProperty(XMLC_COMPILE_JAVA, b);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public boolean isDumpSelected() {
	return readBooleanProperty(XMLC_DUMP, false);
    }

    /**
     * Method declaration
     *
     *
     * @param b
     *
     * @see
     */
    public void setDumpSelected(boolean b) {
	writeBooleanProperty(XMLC_DUMP, b);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public boolean isParseInfoSelected() {
	return readBooleanProperty(XMLC_PARSEINFO, false);
    }

    /**
     * Method declaration
     *
     *
     * @param b
     *
     * @see
     */
    public void setParseInfoSelected(boolean b) {
	writeBooleanProperty(XMLC_PARSEINFO, b);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public boolean isPrintAccessMethodsSelected() {
	return readBooleanProperty(XMLC_PRINTACCESSMETHODS, false);
    }

    /**
     * Method declaration
     *
     *
     * @param b
     *
     * @see
     */
    public void setPrintAccessMethodsSelected(boolean b) {
	writeBooleanProperty(XMLC_PRINTACCESSMETHODS, b);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public boolean isPrintInfoSelected() {
	return readBooleanProperty(XMLC_PRINTINFO, false);
    }

    /**
     * Method declaration
     *
     *
     * @param b
     *
     * @see
     */
    public void setPrintInfoSelected(boolean b) {
	writeBooleanProperty(XMLC_PRINTINFO, b);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public boolean isVerboseSelected() {
	return readBooleanProperty(XMLC_VERBOSE, false);
    }

    /**
     * Method declaration
     *
     *
     * @param b
     *
     * @see
     */
    public void setVerboseSelected(boolean b) {
	writeBooleanProperty(XMLC_VERBOSE, b);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public boolean isKeepJavaSelected() {
	return readBooleanProperty(XMLC_KEEP_JAVA, true);
    }

    /**
     * Method declaration
     *
     *
     * @param b
     *
     * @see
     */
    public void setKeepJavaSelected(boolean b) {
	writeBooleanProperty(XMLC_KEEP_JAVA, b);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public boolean isSaveOutputSelected() {
	return readBooleanProperty(XMLC_SAVE_OUTPUT, true);
    }

    /**
     * Method declaration
     *
     *
     * @param b
     *
     * @see
     */
    public void setSaveOutputSelected(boolean b) {
	writeBooleanProperty(XMLC_SAVE_OUTPUT, b);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public int getMapScope() {
      String strScope;
      int intScope;
      strScope = getProperty(XMLC_MAP_SCOPE);
      intScope = PropUtil.stringToInt(strScope, MAP_SCOPE_ALL);
      return intScope;
    }

    /**
     * Method declaration
     *
     *
     * @param b
     *
     * @see
     */
    public void setMapScope(int i) {
	setProperty(XMLC_MAP_SCOPE, i);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String getOutputFilename() {
	String out = null;

	out = getProperty(XMLC_OUTPUT_FILENAME);

	return out;
    }

    /**
     * Method declaration
     *
     *
     * @param n
     *
     * @see
     */
    public void setOutputFilename(String n) {
	setProperty(XMLC_OUTPUT_FILENAME, n);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String[][] getPackageMap() {
	String[][] map = null;
	String     from = null;
	String     to = null;
	Vector     mapVector = new Vector();

	// we get and put 100 package map's since it is not known how to
	// disappear them from the project once they have been added. Assinging
	// a value of null to the property doesn't delete it from the project,
	// so this way we don't end up with any garbage, non-null properties.
	for (int i = 0; i < 100; i++) {
	    from = getProperty(XMLC_MAP_PACKAGE_FROM + "." + i);
	    to = getProperty(XMLC_MAP_PACKAGE_TO + "." + i);

	    if (from != null && to != null) {
		if (from.trim().length() > 0 && to.trim().length() > 0) {
		    String[] row =
			new String[2];    // note: new 'row' object is needed for each call to addElement

		    row[0] = from;
		    row[1] = to;

		    mapVector.addElement(row);
		}
	    }
	}

	int size = mapVector.size();

	if (size > 0) {
	    String[] row = new String[2];

	    map = new String[size][2];

	    for (int i = 0; i < size; i++) {
		row = (String[]) mapVector.elementAt(i);
		map[i][0] = row[0];
		map[i][1] = row[1];
	    }
	}

	return map;
    }

    /**
     * Method declaration
     *
     *
     * @param map
     *
     * @see
     */
    public void putPackageMap(String[][] map) {
	int    rowCount = map.length;
	String from = new String();
	String to = new String();
	int    i;

	for (i = 0; i < 100; i++) {
	    setProperty(XMLC_MAP_PACKAGE_FROM + "." + i, null);
	    setProperty(XMLC_MAP_PACKAGE_TO + "." + i, null);
	}

	for (i = 0; i < rowCount; i++) {
	    from = map[i][0];
	    to = map[i][1];

	    if (from.trim().length() > 0 && to.trim().length() > 0) {
		setProperty(XMLC_MAP_PACKAGE_FROM + "." + i, from);
		setProperty(XMLC_MAP_PACKAGE_TO + "." + i, to);
	    }
	}
    }

    /**
     * Method declaration
     *
     *
     * @param property
     * @param def
     *
     * @return
     *
     * @see
     */
    private boolean readBooleanProperty(String property, boolean def) {
	boolean flag = def;
	String  in = getProperty(property);

	return PropUtil.stringToBoolean(in, def);
    }

    /**
     * Method declaration
     *
     *
     * @param property
     * @param b
     *
     * @see
     */
    private void writeBooleanProperty(String property, boolean b) {
	String flag = b ? "true" : "false";

	setProperty(property, flag);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String getGenerateToDirectory() {
	String genToFlag = getProperty("GenerateSourceToOutDir");
	String genToDir = null;

	if (genToFlag != null) {
	    if (genToFlag.trim().equalsIgnoreCase("true")) {
		genToDir = getOutputDirectory() + File.separator
			   + "Generated Source";
	    }
	}

	return genToDir;
    }

}



/*--- formatting done in "Sun Java Convention" style on 01-16-2000 ---*/

