/*
 * Enhydra Java Application Server
 * The Initial Developer of the Original Code is Lutris Technologies Inc.
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc.
 * All Rights Reserved.
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License.
 *
 * $Id: OtterXMLCNode.java,v 1.1.2.9 2000/02/13 03:41:10 pmahar Exp $
 */

package org.enhydra.ide.jbuilder.common.node;

import org.enhydra.ide.jbuilder.common.PropUtil;
import org.enhydra.ide.jbuilder.common.node.OtterProject;
import org.enhydra.xml.xmlc.compiler.Parse;
import org.enhydra.xml.xmlc.XMLCException;
import org.enhydra.xml.xmlc.commands.xmlc.XMLCOptions;
import org.enhydra.xml.xmlc.commands.options.OptionsParser;
import org.enhydra.xml.xmlc.commands.options.OptionSet;

import java.io.File;

public class OtterXMLCNode implements OtterHTMLFileNode {
    public final static int CLASS_NAME_DEFAULT = 1;
    public final static int CLASS_NAME_CUSTOM = 2;
    public final static int CLASS_NAME_MAPPED = 3;
    private final String CUSTOM_CLASS_NAME = "enhydra.xmlc.customClassName";
    private final String CLASS_NAME_TYPE   = "enhydra.xmlc.classNameType";
    private final String OPTION_FILENAME   = "enhydra.xmlc.optionFile";

    private boolean parseFailure = false;
    private Exception    exception;
    private Options      options;
    private OtterHTMLFileNode fileNode;

    /**
     * Create a node to use when calling XMLC.
     *
     */
  public OtterXMLCNode(OtterHTMLFileNode f) throws
     XMLCException {
    fileNode = f;
    initOptions();
  }

  public boolean isCompileSelected() {
    return fileNode.isCompileSelected();
  }

  public void setCompileSelected(boolean b) {
    fileNode.setCompileSelected(b);
  }

  public char getFileSeparatorChar() {
    return fileNode.getFileSeparatorChar();
  }

  public String getFilename() {
    return fileNode.getFilename();
  }

  public String getOptionFilename() {
    return getProperty(OPTION_FILENAME);
  }

  public String getJavacDirectory() {
    return fileNode.getJavacDirectory();
  }

  public void setJavacDirectory(String s) {
      fileNode.setJavacDirectory(s);
   }

  public void setOptionFilename(String n) {
    setProperty(OPTION_FILENAME, n);
  }

  public String getCustomClassName() {
    String custom = getProperty(CUSTOM_CLASS_NAME);
    if (custom == null) {
      custom = new String();
    }
    return custom;
  }

  public void setCustomClassName(String n) {
    setProperty(CUSTOM_CLASS_NAME, n);
  }


  public Object getNativeNode() {
    return fileNode.getNativeNode();
  }

  public void setNativeNode(Object o) {
    fileNode.setNativeNode(o);
  }

  public OtterProject getProject() {
    return fileNode.getProject();
  }


  public String getProperty(String n) {
    return fileNode.getProperty(n);
  }

  public void setProperty(String n, String v) {
    fileNode.setProperty(n, v);
  }

  public void setProperty(String n, int v) {
    fileNode.setProperty(n, v);
  }


  public int getClassNameType() {
    // When getting the project scope overrides
    // local type.
    int type;
    int projectScope;
    String stringType = getProperty(CLASS_NAME_TYPE);
    projectScope = getProject().getMapScope();
    type         = PropUtil.stringToInt(stringType, CLASS_NAME_MAPPED);
    switch (type) {
      case CLASS_NAME_DEFAULT:
      case CLASS_NAME_CUSTOM:
           if (projectScope == OtterProject.MAP_SCOPE_ALL) {
              type = CLASS_NAME_MAPPED;
              setProperty(CLASS_NAME_TYPE, type);
           }
           break;
      case CLASS_NAME_MAPPED:
           if (projectScope == OtterProject.MAP_SCOPE_NONE) {
              type = CLASS_NAME_DEFAULT;
              setProperty(CLASS_NAME_TYPE, type);
           }
           break;
    }
    return type;
  }


  public void setClassNameType(int newType) {
    // If conflict exists with project scope,
    // change project scope.
    int oldType       = getClassNameType();
    int projectScope = getProject().getMapScope();
    setProperty(CLASS_NAME_TYPE, newType);
    syncProjectToType(newType);
    if ((newType != oldType) && (newType != CLASS_NAME_CUSTOM)) {
      initClassName();
    }
  }

  private void syncProjectToType(int type) {
    int projectScope = getProject().getMapScope();
    switch (type){
      case CLASS_NAME_CUSTOM:
      case CLASS_NAME_DEFAULT:
           if (projectScope == OtterProject.MAP_SCOPE_ALL) {
               getProject().setMapScope(OtterProject.MAP_SCOPE_SELECTED);
           }
           break;
      case CLASS_NAME_MAPPED:
           if (projectScope == OtterProject.MAP_SCOPE_NONE) {
               getProject().setMapScope(OtterProject.MAP_SCOPE_SELECTED);
           }
          break;
    }
  }


    /**
     * Save Exception information for reporting. This lets exceptions from
     * multiple nodes to be reported in a batch.
     *
     * @param e
     * An exception that occured while invoking XMLC.
     */
    public void setException(Exception e) {
	exception = e;
    }

    /**
     * Get an exception that occured when invoking XMLC. Null if the node
     * has not been compiled or was compiled without error.
     */
    public Exception getException() {
	return exception;
    }

    /**
     * Get the XMLC options selected this HTML file.
     */
    public Options getOptions() {
	return options;
    }

    /**
     * Add the generated Java source file to the IDE project. Do not call
     * this when using Win32 JBuilder as it can't handle the width of source
     * lines generated by XMLC.
     *
     * TODO: Fix this.
     */
    public void addJavaSourceAsNode() {
	if ((options.getJavaClassSource().exists())
		&& (! getProject().isCompileJavaAllowed())) {
           String javaSource = options.getJavaClassSource().getAbsolutePath();
           OtterNodeFactory factory = getProject().getNodeFactory();
           factory.createJavaFileNode(getProject(), fileNode, javaSource);
	}
    }

   public void initProjectOptions() {
	options.setKeep(getProject().isKeepJavaSelected());
	options.setNoCompile(false); //!getProject().isCompileJavaSelected());

	// loging options
	options.setParser(Parse.HTML_SWING_PARSER);
        options.setParseInfo(getProject().isParseInfoSelected());
        options.setDump(getProject().isDumpSelected());
	options.setVerbose(getProject().isVerboseSelected());
	options.setPrintAccessMethods(getProject().isPrintAccessMethodsSelected());

	// Standard JAVAC options
	// compilition options
        if (fileNode.getProject().isCompileJavaAllowed()) {
           options.setJavac("bcjhide.exe");  // Win32 JBuilder and JDeveloper
	}
   }


    /**
     * Initialize the XMLC options.
     *
     * @throws XMLCException
     * Thrown when options are invalid.
     *
     * TODO: Mark is changing stuff that might change this method.
     * TODO: Add support of xmlc file to do advanced options (-implements,
     * -urlmapping, -urlsetting, javacflag, -javacopt, and -docout).
     */
    private void initOptions() throws XMLCException {
    	if (fileNode.getProperty(fileNode.COMPILE_SELECTED) == null) {
           String projFile = getProject().getFilename().trim();
           String nodeFile = getFilename().trim();
           int index = projFile.indexOf(".");
           if (index > 0) {
             projFile = projFile.substring(0, index).trim();
           }
           index = nodeFile.indexOf(".");
           if (index > 0) {
             nodeFile = nodeFile.substring(0, index).trim();
           }
           fileNode.setCompileSelected(!nodeFile.equalsIgnoreCase(projFile));
	}

	options = new Options();
	initClassName();
        initProjectOptions();
        File sourceDoc = null;
        sourceDoc = new File(getFilename());
        options.setSourceDocument(sourceDoc, true);
        initJavaSourceFile();
        options.complete();
/*
// Reinstate for supporting for .xmlc files at the node level.
        if ((getOptionFilename() != null) && (! parseFailure)) {
          OptionsParser op = new OptionsParser(); // Needs options in constructor in 3.0
          String args[] = new String[1];
          args[0] = getOptionFilename();
          try {
            op.parse(options, args); // Only pass args in 3.0
          } catch (java.io.IOException e) {
            e.printStackTrace();
            parseFailure = true;
            initOptions();
          }
        }
*/
      }

     private void initJavaSourceFile() {
	StringBuffer filename = new StringBuffer();
        int index = -1;
        filename.append(getGenToDirectory().toString());
        filename.append("" + getProject().getFileSeparatorChar());
        String className = options.getClassName();
        if (options.getPackageName() != null) {
          index = className.indexOf(options.getPackageName());
        }
        if (index > -1) {
          className = className.substring(options.getPackageName().length()+1);
        }
        index = className.lastIndexOf(".");
        if (index > -1) {
          className = className.substring(index + 1);
        }
        filename.append(className);
	filename.append(".java");
	File f = new File(filename.toString());
	options.setJavaClassSource(f);
    }

    /**
     * Get the generate to directory and create the directory if needed.
     * This is where the Java source files are created.
     */
    private File getGenToDirectory() {
	String genTo = getGenTo();
	File   genDir = new File(genTo);

	if (genDir.exists()) {
	    if (!genDir.isDirectory()) {
		System.err.println("Unable to write to directory: " + genTo);
	    }
	} else {
	    genDir.mkdirs();
	}

	return genDir;
    }

    /**
     * Get the generate to directory as a string. This is where XMLC will create
     * Java source files.
     */
    private String getGenTo() {
	String genTo = getProject().getGenerateToDirectory();
	File   sourceFile = null;

	if (genTo == null) {
	    sourceFile = new File(getFilename());
	    genTo = sourceFile.getParent();
	}

	return genTo;
    }

    /**
     * Create a class name for the given HTML file and map the
     * package name as needed. Mapping allows users to  specify
     * a package name to use for all the HTML files in a directory.
     */
    private void initClassName() {
        boolean  usedCustom = false;
	String   className = new String();
	String   lowername = getFilename().toLowerCase().trim();
	String[] path = getProject().getSourcePathArray();
	Mapper   mapper = null;

	for (int i = 0; i < path.length; i++) {
            String lowerPath = path[i].toLowerCase().trim();
            if (getClassNameType() == CLASS_NAME_CUSTOM &&
                           getCustomClassName().trim().length() > 0) {
                    className = getCustomClassName();
                    usedCustom = true;
              break;
	    } else if (lowername.startsWith(lowerPath)) {
		mapper = new Mapper(path[i], lowername);
		if (mapper.isMapped()) {
		    className = mapper.getClassName();
                } else {
		    className =
			getFilename().trim().substring(path[i].length(),
				    lowername.lastIndexOf("."));
		}

		className =
		    className.replace(getFileSeparatorChar(),'.');
		if (className.startsWith(".")) {
		    className = className.substring(1);
		}
		break;
	    }
	}

        if (! usedCustom) {
           int at = -1;
            if (className.trim().length()==0) {
              // not in path, create packageless default name.
              at = lowername.lastIndexOf(getFileSeparatorChar());
              className = lowername.substring(at + 1);
              at = className.lastIndexOf(".");
              className = className.substring(0, at);
            }

            at = getFilename().lastIndexOf(getFileSeparatorChar());
            String baseName = getFilename().substring(at + 1,
                          getFilename().lastIndexOf("."));
	    at = className.lastIndexOf(".");
	    className = className.substring(0, at + 1)
                      + baseName + "HTML";
        }
        options.setClassName(className);
    }

    /**
     * Get the class file that we expect after XMLC has generated
     * a java file and called javac to compile it.
     */
    public File getClassFile() {
	String cfName = getProject().getOutputDirectory() +
                      "" + getProject().getFileSeparatorChar()
			+ options.getClassName();

	cfName = cfName.replace('.', getProject().getFileSeparatorChar());
	cfName = cfName + ".class";

	File cf = new File(cfName);

	return cf;
    }

    /**
     * True if we can find the class file that we expect XMLC to create.
     * This is used to in reporting compilation results.
     */
    public boolean isNewClassFound() {
	boolean newFound = false;

	if (!options.getNoCompile()) {
	    newFound = getClassFile().exists();
	}

	return newFound;
    }

    /**
     * True if we can find the Java source file that we expect XMLC to create.
     * This is used to in reporting compilation results.
     */
    public boolean isNewJavaFound() {
	boolean newFound = false;

	if (options.getKeep()) {
	    newFound = options.getJavaClassSource().exists();
	}

	return newFound;
    }

    /**
     * Delete the class and java files that we expect to get recreated by XMLC.
     */
    private void clearFiles() {
	if (options.getKeep() && options.getJavaClassSource().exists()) {
	    options.getJavaClassSource().delete();
	}

	File cf = getClassFile();

	if ((!options.getNoCompile()) && (cf.exists())) {
	    cf.delete();
	}
    }

    public void preCompile() {
      fileNode.preCompile();
    }

    /**
     * Class declaration
     *
     *
     * @author
     * @version %I%, %G%
     */
    class Mapper {
	private String  path = new String();
	private String  filename = new String();
	private boolean mapped = false;
	private String  lowerFrom = new String();
	private String  toPackage = new String();

	/**
	 * Constructor declaration
	 *
	 *
	 * @param p
	 * @param f
	 *
	 * @see
	 */
	public Mapper(String p, String f) {
	    String[][] map = null;
            boolean useMap = false;
	    if (getProject().getMapScope() == OtterProject.MAP_SCOPE_ALL) {
              useMap = true;
            } else if (getProject().getMapScope() == OtterProject.MAP_SCOPE_SELECTED
                   && getClassNameType() == CLASS_NAME_MAPPED) {
              useMap = true;
            }
	    if (useMap) {
		map = getProject().getPackageMap();
            }

	    this.path     = p.trim().toLowerCase();
	    this.filename = f.trim().toLowerCase();


            if (map != null) {
		if (!path.endsWith("" + getProject().getFileSeparatorChar())) {
		    path = path + "" + getProject().getFileSeparatorChar();
		}


		for (int i = 0; i < map.length; i++) {
		    lowerFrom = map[i][0].trim().toLowerCase();
                    lowerFrom = lowerFrom.replace(File.separatorChar, getProject().getFileSeparatorChar());
		    toPackage = map[i][1].trim();
                    filename  = filename.replace(File.separatorChar, getProject().getFileSeparatorChar());
		    if (filename.startsWith(lowerFrom)) {
			mapped = true;

			break;
		    }
		}
	    }
	}

	/**
	 * Method declaration
	 *
	 *
	 * @return
	 *
	 * @see
	 */
	private boolean isMapped() {
	    return mapped;
	}

	/**
	 * Method declaration
	 *
	 *
	 * @return
	 *
	 * @see
	 */
	private String getClassName() {
	    String mappedClass = null;

	    if (mapped) {
		mappedClass = toPackage
			      + filename.substring(lowerFrom.length(),
						   filename.lastIndexOf("."));
	    }

	    return mappedClass;
	}

    }

    /**
     * Class declaration
     *
     *
     * @author
     * @version %I%, %G%
     */
    public class Options extends XMLCOptions implements OptionSet {
      private File javaClassSource = null;
      private File sourceDoc = null;

      public void setJavaClassSource(File f) {
        javaClassSource = f;
      }

      public File getJavaClassSource() {
        return javaClassSource;
      }

      public void setSourceDocument(File f, boolean real) {
        if (real) {
          sourceDoc = f;
          setSourceDocument(sourceDoc);
        }
      }

      public void setSourceDocument(File f) {
        super.setSourceDocument(sourceDoc);
      }

      public File getSourceDocument() {
        return sourceDoc;
      }
  }

}


