/*
 * Enhydra Java Application Server
 * The Initial Developer of the Original Code is Lutris Technologies Inc.
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc.
 * All Rights Reserved.
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License.
 *
 * $Id: EnhydraAppMaker.java,v 1.1.2.10 2000/02/13 09:50:27 pmahar Exp $
 */

package org.enhydra.ide.jbuilder.common.wizard.newapp;

import org.enhydra.devsys.wizard.newapp.NewAppWizard;
import org.enhydra.ide.jbuilder.common.node.OtterProject;
import org.enhydra.ide.jbuilder.common.node.OtterFolderNode;
import org.enhydra.ide.jbuilder.common.node.OtterHTMLFileNode;
import org.enhydra.ide.jbuilder.common.node.OtterJavaFileNode;
import org.enhydra.ide.jbuilder.common.node.OtterTextFileNode;
import org.enhydra.ide.jbuilder.common.node.OtterNodeFactory;
import java.io.File;
import java.io.FilenameFilter;

/**
 * Class declaration
 *
 *
 * @author
 * @version %I%, %G%
 */
public class EnhydraAppMaker extends NewAppWizard {
    private OtterProject project = null;
    private static final String GALLERY_NAME = "Enhydra Application";

    public static String getGalleryName() {
      return GALLERY_NAME;
    }
    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public OtterProject getProject() {
	return project;
    }

    /**
     * Method declaration
     *
     *
     * @param p
     *
     * @see
     */
    public void setProject(OtterProject p) {
	String[] sourcePath;
        String fixedName;
        int index = -1;

	project = p;


        fixedName = project.getName();
        index = fixedName.indexOf(".");
        if (index > -1) {
          fixedName = fixedName.substring(0, index);
        }
       	setAppName(fixedName);
	setPackageName(fixedName);

	sourcePath = project.getSourcePathArray();
    }

    /**
     * Method declaration
     *
     *
     * @see
     */
    public void addAppToProject() {
	OtterFolderNode otterFolder;
        OtterHTMLFileNode html;

	otterFolder = addPresentationFolder();

	addMediaFolder(otterFolder);
	addFolderToProject("business");
	addFolderToProject("data");
	addOutputFolder();
	addJavaToProject(getAppJava().getAbsolutePath());
	html = addHTMLToProject(getReadme().getAbsolutePath());
        html.setCompileSelected(false);

        addJavaToProject(getStartMultiServer().getAbsolutePath());
        addConfFolder();
	configureProjectProperties();
    }

    public File getAppJava() {
       StringBuffer path = new StringBuffer();
       path.append(getPackageDir());
       path.append(File.separator);
       path.append(getAppName().substring(0,1).toUpperCase());
       path.append(getAppName().substring(1).toLowerCase());
       path.append(".java");
       File appJava = new File(path.toString());
       return appJava;
    }

    /**
     * Method declaration
     *
     *
     * @see
     */
    public void configureProjectProperties() {
	setSourcePath();
	project.setProperty("Libraries", "Enhydra 2.3");
    }

    /**
     * Method declaration
     *
     *
     * @see
     */
    private void setSourcePath() {
	String[] path = project.getSourcePathArray();
	String[] newPath;
	File     current;
	String[] javaList;
	boolean  javaFound = false;

	for (int i = 0; i < path.length; i++) {
	    current = new File(path[i]);
	    javaList = current.list(new FilenameFilter() {

		/**
		 * Method declaration
		 *
		 *
		 * @param dir
		 * @param name
		 *
		 * @return
		 *
		 * @see
		 */
		public boolean accept(File dir, String name) {
		    return name.trim().toLowerCase().endsWith("java");
		}

	    });

	    if ((javaList != null) && (javaList.length > 0)) {
		javaFound = true;
	    }
	}

	if (javaFound) {
	    newPath = new String[path.length + 1];
	    newPath[0] = getAppDir();

	    for (int i = 1; i < (path.length + 1); i++) {
		newPath[i] = path[i - 1];
	    }
	} else {
	    newPath = new String[1];
	    newPath[0] = getAppDir();
	}

	project.setSourcePathArray(newPath);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    private OtterFolderNode addPresentationFolder() {
	OtterNodeFactory factory = project.getNodeFactory();
	File		 f = null;

	// add Presentation Package
	String		 presentationPackage = getPackageName()
					       + ".presentation";
	String		 presentationDir = getPackageDir() + File.separator
					   + "presentation";
	OtterFolderNode  folder = factory.createFolderNode(project, project,
		"presentation");

	// Add welcome.html to folder
	f = new File(presentationDir + File.separator + "Welcome.html");

	if (f.exists()) {
	    OtterHTMLFileNode html = factory.createHTMLFileNode(project,
		    folder, f.getAbsolutePath());
            html.setCompileSelected(true);
	}

	// Add welcome.java to folder
	f = new File(presentationDir + File.separator + "Welcome.java");

	if (f.exists()) {
	    factory.createJavaFileNode(project, folder, f.getAbsolutePath());
	}

	// Add redirect.java to folder
	f = new File(presentationDir + File.separator + "Redirect.java");

	if (f.exists()) {
	    factory.createJavaFileNode(project, folder, f.getAbsolutePath());
	}

	return folder;
    }

    /**
     * Method declaration
     *
     *
     * @param presentation
     *
     * @return
     *
     * @see
     */
    private OtterFolderNode addMediaFolder(OtterFolderNode presentation) {

	// Add presentation.media folder
	OtterFolderNode  folder;
	OtterNodeFactory factory = project.getNodeFactory();
	File		 f;

	folder = factory.createFolderNode(project, presentation, "media");

	// Add enhdyra.gif to media
	String presentationDir = getPackageDir() + File.separator
				 + "presentation";

	f = new File(presentationDir + File.separator + "media"
		     + File.separator + "Enhydra.gif");

	if (f.exists()) {
	    factory.createImageFileNode(project, folder, f.getAbsolutePath());
	}

	return folder;
    }

    /**
     * Method declaration
     *
     *
     * @param name
     *
     * @return
     *
     * @see
     */
    private OtterFolderNode addFolderToProject(String name) {

	// business layer
	OtterFolderNode  folder;
	OtterNodeFactory factory = project.getNodeFactory();

	folder = factory.createFolderNode(project, project, name);

	return folder;
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    private OtterFolderNode addOutputFolder() {

	// output log
	OtterFolderNode  folder;
	OtterNodeFactory factory = project.getNodeFactory();

	folder = factory.createFolderNode(project, project, "output");

	if (getLogFile().exists()) {
	    factory.createTextFileNode(project, folder,
				       getLogFile().getAbsolutePath());
	}

	return folder;
    }

    private OtterFolderNode addConfFolder() {
	OtterFolderNode  folder;
	OtterNodeFactory factory = project.getNodeFactory();

	folder = factory.createFolderNode(project, project, "conf");

	if (getMultiServerConf().exists()) {
	    factory.createTextFileNode(project, folder,
				       getMultiServerConf().getAbsolutePath());
	}
	if (getAppConf().exists()) {
	    factory.createTextFileNode(project, folder,
				       getAppConf().getAbsolutePath());
	}

	return folder;
    }


    /**
     * Method declaration
     *
     *
     * @param file
     *
     * @return
     *
     * @see
     */
    private OtterJavaFileNode addJavaToProject(String file) {
	OtterJavaFileNode java = null;
	OtterNodeFactory  factory = project.getNodeFactory();
	File		  f = new File(file);

	if (f.exists()) {
	    java = factory.createJavaFileNode(project, project, file);
	}

	return java;
    }

    /**
     * Method declaration
     *
     *
     * @param file
     *
     * @return
     *
     * @see
     */
    private OtterHTMLFileNode addHTMLToProject(String file) {
	OtterHTMLFileNode html = null;
	OtterNodeFactory  factory = project.getNodeFactory();
	File		  f = new File(file);

	if (f.exists()) {
	    html = factory.createHTMLFileNode(project, project, file);
	}

	return html;
    }

    private OtterTextFileNode addTextToProject(String file) {
	OtterTextFileNode java = null;
	OtterNodeFactory  factory = project.getNodeFactory();
	File		  f = new File(file);

	if (f.exists()) {
	    java = factory.createTextFileNode(project, project, file);
	}

	return java;
    }


}

