/*
 * Enhydra Java Application Server
 * The Initial Developer of the Original Code is Lutris Technologies Inc.
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc.
 * All Rights Reserved.
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License.
 *
 * $Id: CompileDialog.java,v 1.1.2.7 2000/02/13 03:41:11 pmahar Exp $
 */

package org.enhydra.ide.jbuilder.common.wizard.xmlc;

import org.enhydra.ide.jbuilder.common.node.OtterProject;
import org.enhydra.ide.jbuilder.common.node.OtterXMLCNode;
import org.enhydra.devsys.swing.GridBagConstraintsX;
import org.enhydra.devsys.swing.SwingUtil;
import javax.swing.*;
import java.beans.*;
import java.awt.*;
import java.awt.event.*;

/**
 * This class defines the main dialog for the XMLC Wizard.
 *
 * @author
 * Paul Mahar
 * @version %I%, %G%
 */
public class CompileDialog extends JDialog {
    public static final String  XMLC_CLASS =
	"org.enhydra.xml.xmlc.commands.xmlc.XMLC";
    private boolean		standalone = false;
    private OtterXMLCNode[]	nodes = null;
    private OtterProject	project = null;
    private LocalButtonListener buttonListener = null;
    private JPanel		panelMain;
    private JPanel		panelTab;
    private JPanel		panelButton;
    private JPanel		panelBottom;
    private JPanel		panelFiller2;
    private GridBagLayout       layoutMain;
    private GridBagLayout       layoutOption;
    private GridBagLayout       layoutButton;
    private GridBagLayout       layoutBottom;
    private XMLCPanel		xmlcPanel;
    private OptionPanel		optionPanel;
    private OutputPanel		outputPanel;
    private JTabbedPane		tabCompiler;
    private JButton		buttonCompile;
    private JButton		buttonClose;

    /**
     * Create the dialog as a child of a frame.
     *
     * @param owner
     * The parent Frame used when the dialog is modal.
     * @param title
     * The title for the dialog.
     * @param modal
     * Normally this is set to true for wizards.
     */
    public CompileDialog(Frame owner, String title, boolean modal) {
	super(owner, title, modal);

	construct();
    }

    /**
     * Create the dialog as a child of another dialog.
     *
     * @param owner
     * The parent Dialog used when the dialog is modal.
     * @param title
     * The title for the dialog.
     * @param modal
     * Normally this is set to true for wizards.
     */
    public CompileDialog(Dialog owner, String title, boolean modal) {
	super(owner, title, modal);

	construct();
    }

    /**
     * Overloaded method that will center the dialog prior to making it
     * visible.
     */
    public void show() {
	setResizable(false);

	Point cPoint;

	invalidate();
	doLayout();
	pack();

	cPoint = SwingUtil.getCenteringPoint(getSize());

	setLocation(cPoint);
	super.show();
    }

    /**
     * Remove all object references.
     */
    public void clearAll() {
	layoutMain.removeLayoutComponent(panelTab);
	panelTab.removeAll();
	panelMain.removeAll();

	layoutOption = null;
	layoutMain = null;
	panelTab = null;
	panelMain = null;

	setTitle(null);
    }

    /**
     * Get source files that can be selected for compilation.
     */
    public OtterXMLCNode[] getNodes() {
	return nodes;
    }

    /**
     * Set the source files that can be selected for compilation
     */
    public void setNodes(OtterXMLCNode[] n) {
	nodes = n;

	xmlcPanel.setNodes(nodes);
    }

    /**
     * Get the IDE project containing the HTML files.
     */
    public OtterProject getProject() {
	return project;
    }

    /**
     * Set the IDE project containing the HTML files.
     */
    public void setProject(OtterProject p) {
	project = p;

	xmlcPanel.setProject(project);
	optionPanel.setProject(project);
	outputPanel.setProject(project);
    }

    /**
     * See if the copmile button is enabled.
     */
    protected boolean isCompileButtonEnabled() {
	return buttonCompile.isEnabled();
    }

    /**
     * Allow user to invoke XMLC.
     *
     * @param enable
     * True if you want the compile button enabled.
     */
    protected void setCompileButtonEnabled(boolean enable) {
	buttonCompile.setEnabled(true);
    }

    /**
     * Close the dialog and exit the VM if this wizard
     * was run as a standalone application.
     */
    protected void closeDialog() {
	setVisible(false);
	removeAll();
	dispose();

	if (standalone) {
	    System.exit(0);
	}
    }

    /**
     * Select the output to show compilation progress. This
     * is called when XMLC is invoked.
     */
    private void selectOutputTab() {
	Runnable runSwing = new Runnable() {

	    /**
	     * Method declaration
	     *
	     *
	     * @see
	     */
	    public void run() {
		tabCompiler.setSelectedComponent(outputPanel);
		SwingUtilities.windowForComponent(tabCompiler).repaint();
	    }

	};

	try {
	    if (SwingUtilities.isEventDispatchThread()) {
		runSwing.run();
	    } else {
		SwingUtilities.invokeAndWait(runSwing);
	    }
	} catch (Exception e) {
	    e.printStackTrace();
	}

	try {
	    Thread.sleep(500);
	} catch (InterruptedException e) {

	    //
	}
    }

    /**
     * Scroll to the bottom of the output pane to see
     * the latest logging messages.
     */
    private void scrollOutputToBottom() {
	outputPanel.scrollToBottom();
    }

    /**
     * Construct the object. This is called from both constructors.
     */
    private void construct() {
	try {
	    jbInit();
	    pmInit();
	    pack();
	} catch (Exception ex) {
	    ex.printStackTrace();
	}
    }

    /**
     * Add listeners and other items that are not generated by
     * the JBuilder designer.
     */
    private void pmInit() {
	tabCompiler.add("Pages", xmlcPanel);
	tabCompiler.add("Options", optionPanel);
	tabCompiler.add("Output", outputPanel);
	xmlcPanel.addWriteListener(outputPanel);

	buttonListener = new LocalButtonListener();

	buttonCompile.addActionListener(buttonListener);
	buttonClose.addActionListener(buttonListener);
    }

    /**
     * UI code generated by JBuilder.
     * <P><I>Do not modify the signature of this method.</I></P>
     */
    private void jbInit() throws Exception {
	panelMain = (JPanel) Beans.instantiate(getClass().getClassLoader(),
					       JPanel.class.getName());
	panelTab = (JPanel) Beans.instantiate(getClass().getClassLoader(),
					      JPanel.class.getName());
	panelBottom = (JPanel) Beans.instantiate(getClass().getClassLoader(),
						 JPanel.class.getName());
	panelButton = (JPanel) Beans.instantiate(getClass().getClassLoader(),
						 JPanel.class.getName());
	panelFiller2 = (JPanel) Beans.instantiate(getClass().getClassLoader(),
						  JPanel.class.getName());
	layoutMain =
	    (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
					      GridBagLayout.class.getName());
	layoutMain =
	    (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
					      GridBagLayout.class.getName());
	layoutOption =
	    (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
					      GridBagLayout.class.getName());
	layoutButton =
	    (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
					      GridBagLayout.class.getName());
	layoutBottom =
	    (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
					      GridBagLayout.class.getName());
	xmlcPanel = (XMLCPanel) Beans.instantiate(getClass().getClassLoader(),
						  XMLCPanel.class.getName());
	optionPanel =
	    (OptionPanel) Beans.instantiate(getClass().getClassLoader(),
					    OptionPanel.class.getName());
	outputPanel =
	    (OutputPanel) Beans.instantiate(getClass().getClassLoader(),
					    OutputPanel.class.getName());
	tabCompiler =
	    (JTabbedPane) Beans.instantiate(getClass().getClassLoader(),
					    JTabbedPane.class.getName());
	buttonCompile =
	    (JButton) Beans.instantiate(getClass().getClassLoader(),
					JButton.class.getName());
	buttonClose = (JButton) Beans.instantiate(getClass().getClassLoader(),
						  JButton.class.getName());

	buttonCompile.setText("Compile");
	buttonCompile.setEnabled(false);
	buttonClose.setText("Close");
	panelTab.setLayout(layoutOption);
	panelTab.add(tabCompiler,
		     new GridBagConstraintsX(0, 0, 1, 1, 1.0, 1.0,
					    GridBagConstraints.CENTER,
					    GridBagConstraints.BOTH,
					    new Insets(5, 5, 5, 5), 5, 5));
	panelButton.setLayout(layoutButton);
	panelButton.add(buttonCompile,
			new GridBagConstraintsX(1, 0, 1, 4, 0.5, 0.5,
					       GridBagConstraints.CENTER,
					       GridBagConstraints.NONE,
					       new Insets(5, 5, 5, 5), 0, 0));
	panelButton.add(buttonClose,
			new GridBagConstraintsX(2, 0, 1, 4, 0.5, 0.5,
					       GridBagConstraints.CENTER,
					       GridBagConstraints.NONE,
					       new Insets(5, 5, 5, 5), 0, 0));
	panelBottom.setLayout(layoutBottom);
	panelBottom.add(panelFiller2,
			new GridBagConstraintsX(0, 0, 1, 1, 0.4, 0.4,
					       GridBagConstraints.CENTER,
					       GridBagConstraints.BOTH,
					       new Insets(0, 0, 0, 0), 0, 0));
	panelBottom.add(panelButton,
			new GridBagConstraintsX(1, 0, 1, 4, 0.6, 0.6,
					       GridBagConstraints.EAST,
					       GridBagConstraints.NONE,
					       new Insets(0, 0, 0, 0), 0, 0));
	panelMain.setLayout(layoutMain);
	panelMain.add(panelTab,
		      new GridBagConstraintsX(0, 0, 1, 1, 0.75, 0.75,
					     GridBagConstraints.CENTER,
					     GridBagConstraints.BOTH,
					     new Insets(0, 0, 0, 0), 0, 0));
	panelMain.add(panelBottom,
		      new GridBagConstraintsX(0, 1, 1, 1, 0.25, 0.25,
					     GridBagConstraints.CENTER,
					     GridBagConstraints.BOTH,
					     new Insets(0, 0, 0, 0), 0, 0));
	this.getContentPane().add(panelMain);
    }

    /**
     * This inner class defines the actions for buttons in the wizard.
     *
     * @author
     * Paul Mahar
     * @version %I%, %G%
     */
    class LocalButtonListener implements ActionListener {

	/**
	 * ActionListener implementation.
	 *
	 * @param event
	 * Event that triggered this listener.
	 *
	 */
	public void actionPerformed(ActionEvent event) {
	    Object source = event.getSource();

	    if (source == buttonCompile) {
		selectOutputTab();
                xmlcPanel.setProject(optionPanel.getProject());
		xmlcPanel.startCompile();
	    } else if (source == buttonClose) {
		closeDialog();
	    }
	}

    }

}



/*--- formatting done in "Sun Java Convention" style on 01-16-2000 ---*/

